@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 * Copyright (C) {1999-2001}, International Business Machines Corporation and others. All Rights Reserved.
 **********************************************************************
@@ -8,6 +10,11 @@
 #ifndef RBT_RULE_H
 #define RBT_RULE_H
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
+#include "unicode/uobject.h"
 #include "unicode/unistr.h"
 #include "unicode/utrans.h"
 #include "unicode/unimatch.h"
@@ -47,17 +54,7 @@ class UnicodeFunctor;
  *
  * @author Alan Liu
  */
-class TransliterationRule {
-
-public:
-
-    /**
-     * The character at index i, where i < contextStart || i >= contextLimit,
-     * is ETHER.  This allows explicit matching by rules and UnicodeSets
-     * of text outside the context.  In traditional terms, this allows anchoring
-     * at the start and/or end.
-     */
-    static const UChar ETHER;
+class TransliterationRule : public UMemory {
 
 private:
 
@@ -153,32 +150,34 @@ public:
     /**
      * Construct a new rule with the given input, output text, and other
      * attributes.  A cursor position may be specified for the output text.
-     * @param input input string, including key and optional ante and
-     * post context
+     * @param input          input string, including key and optional ante and
+     *                       post context.
      * @param anteContextPos offset into input to end of ante context, or -1 if
-     * none.  Must be <= input.length() if not -1.
+     *                       none.  Must be <= input.length() if not -1.
      * @param postContextPos offset into input to start of post context, or -1
-     * if none.  Must be <= input.length() if not -1, and must be >=
-     * anteContextPos.
-     * @param output output string
+     *                       if none.  Must be <= input.length() if not -1, and must be >=
+     *                       anteContextPos.
+     * @param outputStr      output string.
      * @param cursorPosition offset into output at which cursor is located, or -1 if
-     * none.  If less than zero, then the cursor is placed after the
-     * <code>output</code>; that is, -1 is equivalent to
-     * <code>output.length()</code>.  If greater than
-     * <code>output.length()</code> then an exception is thrown.
-     * @param cursorOffset an offset to be added to cursorPos to position the
-     * cursor either in the ante context, if < 0, or in the post context, if >
-     * 0.  For example, the rule "abc{def} > | @@@ xyz;" changes "def" to
-     * "xyz" and moves the cursor to before "a".  It would have a cursorOffset
-     * of -3.
-     * @param segs array of UnicodeMatcher corresponding to input pattern
-     * segments, or null if there are none.  The array itself is adopted,
-     * but the pointers within it are not.
-     * @param segsCount number of elements in segs[]
-     * @param anchorStart TRUE if the the rule is anchored on the left to
-     * the context start
-     * @param anchorEnd TRUE if the rule is anchored on the right to the
-     * context limit
+     *                       none.  If less than zero, then the cursor is placed after the
+     *                       <code>output</code>; that is, -1 is equivalent to
+     *                       <code>output.length()</code>.  If greater than
+     *                       <code>output.length()</code> then an exception is thrown.
+     * @param cursorOffset   an offset to be added to cursorPos to position the
+     *                       cursor either in the ante context, if < 0, or in the post context, if >
+     *                       0.  For example, the rule "abc{def} > | @@@ xyz;" changes "def" to
+     *                       "xyz" and moves the cursor to before "a".  It would have a cursorOffset
+     *                       of -3.
+     * @param segs           array of UnicodeMatcher corresponding to input pattern
+     *                       segments, or null if there are none.  The array itself is adopted,
+     *                       but the pointers within it are not.
+     * @param segsCount      number of elements in segs[].
+     * @param anchorStart    true if the the rule is anchored on the left to
+     *                       the context start.
+     * @param anchorEnd      true if the rule is anchored on the right to the
+     *                       context limit.
+     * @param data           the rule data.
+     * @param status         Output parameter filled in with success or failure status.
      */
     TransliterationRule(const UnicodeString& input,
                         int32_t anteContextPos, int32_t postContextPos,
@@ -192,6 +191,7 @@ public:
 
     /**
      * Copy constructor.
+     * @param other    the object to be copied.
      */
     TransliterationRule(TransliterationRule& other);
 
@@ -203,6 +203,7 @@ public:
     /**
      * Change the data object that this rule belongs to.  Used
      * internally by the TransliterationRuleData copy constructor.
+     * @param data    the new data value to be set.
      */
     void setData(const TransliterationRuleData* data);
 
@@ -214,6 +215,7 @@ public:
      * there is a start anchor.  The one character anchor gap is
      * needed to make repeated incremental transliteration with
      * anchors work.
+     * @return    the preceding context length.
      */
     virtual int32_t getContextLength(void) const;
 
@@ -222,6 +224,7 @@ public:
      * This is the low byte of the first character of the key,
      * unless the first character of the key is a set.  If it's a
      * set, or otherwise can match multiple keys, the index value is -1.
+     * @return    8-bit index value for this rule.
      */
     int16_t getIndexValue() const;
 
@@ -234,6 +237,8 @@ public:
      * contains any character with a low byte equal to the index
      * value.  If the rule contains only ante context, as in foo)>bar,
      * then it will match any key.
+     * @param v    the given index value.
+     * @return     true if this rule matches the given index value.
      */
     UBool matchesIndexValue(uint8_t v) const;
 
@@ -242,6 +247,8 @@ public:
      * r1 matches any input string that r2 matches.  If r1 masks r2 and r2 masks
      * r1 then r1 == r2.  Examples: "a>x" masks "ab>y".  "a>x" masks "a[b]>y".
      * "[c]a>x" masks "[dc]a>y".
+     * @param r2  the given rule to be compared with.
+     * @return    true if this rule masks 'r2'
      */
     virtual UBool masks(const TransliterationRule& r2) const;
 
@@ -260,11 +267,11 @@ public:
      * 
      * @param text the text
      * @param pos the position indices
-     * @param incremental if TRUE, test for partial matches that may
+     * @param incremental if true, test for partial matches that may
      * be completed by additional text inserted at pos.limit.
      * @return one of <code>U_MISMATCH</code>,
      * <code>U_PARTIAL_MATCH</code>, or <code>U_MATCH</code>.  If
-     * incremental is FALSE then U_PARTIAL_MATCH will not be returned.
+     * incremental is false then U_PARTIAL_MATCH will not be returned.
      */
     UMatchDegree matchAndReplace(Replaceable& text,
                                  UTransPosition& pos,
@@ -276,11 +283,28 @@ public:
      */
     virtual UnicodeString& toRule(UnicodeString& pat,
                                   UBool escapeUnprintable) const;
+
+    /**
+     * Union the set of all characters that may be modified by this rule
+     * into the given set.
+     */
+    void addSourceSetTo(UnicodeSet& toUnionTo) const;
+
+    /**
+     * Union the set of all characters that may be emitted by this rule
+     * into the given set.
+     */
+    void addTargetSetTo(UnicodeSet& toUnionTo) const;
+
  private:
 
     friend class StringMatcher;
+
+    TransliterationRule &operator=(const TransliterationRule &other); // forbid copying of this class
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 #endif
