@@ -67,7 +67,7 @@ void TestTwoDigitYearDSTParse()
             myErrorName(status) );
     }
     else {
-        log_verbose("PASS: creating dateformat using udat_openPattern() succesful\n");
+        log_verbose("PASS: creating dateformat using udat_openPattern() successful\n");
     
         u_uastrcpy(pattern, "dd-MMM-yy h:mm:ss 'o''clock' a z");
         fmt= udat_open(UDAT_PATTERN,UDAT_PATTERN,"en_US", NULL, 0,pattern, u_strlen(pattern), &status);
@@ -135,7 +135,7 @@ void TestPartialParse994()
     pos=0;
     d = udat_parse(f, s, u_strlen(s), &pos, &status);
     if(U_FAILURE(status)) {
-      log_data_err("FAIL: could not parse - exitting");
+      log_data_err("FAIL: could not parse - exiting");
       return;
     }
     fmtChars = myDateFormat(f, d);
