@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2009, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -19,7 +21,11 @@
  */
 
 #include <stdlib.h>
+
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
 #include "unicode/uloc.h"
 #include "cintltst.h"
@@ -69,7 +75,7 @@ const static UCollationResult results[] = {
     UCOL_GREATER,
     /* test priamry > 8 */
     UCOL_LESS,
-    UCOL_LESS, /*Turkish translator made a primary differnce between dotted and dotless I */
+    UCOL_LESS, /*Turkish translator made a primary difference between dotted and dotless I */
     UCOL_GREATER
 };
 
@@ -92,7 +98,7 @@ static void TestTertiary( )
     UErrorCode status = U_ZERO_ERROR;
     myCollation = ucol_open("tr", &status);
     if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n", myErrorName(status));
         return;
     }
     log_verbose("Testing Turkish Collation with Tertiary strength\n");
@@ -112,7 +118,7 @@ static void TestPrimary()
     UErrorCode status = U_ZERO_ERROR;
     myCollation = ucol_open("tr", &status);
     if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n", myErrorName(status));
         return;
     }
     log_verbose("Testing Turkish Collation with Primary strength\n");
@@ -124,3 +130,4 @@ static void TestPrimary()
     ucol_close(myCollation);
 }
 
+#endif /* #if !UCONFIG_NO_COLLATION */
