@@ -419,6 +419,10 @@ void TestUScriptCodeAPI(){
             "Hanifi_Rohingya", "Sogdian", "Old_Sogdian",
             // new in ICU 64
             "Elymaic", "Nyiakeng_Puachue_Hmong", "Nandinagari", "Wancho",
+            // new in ICU 66
+            "Chorasmian", "Dives_Akuru", "Khitan_Small_Script", "Yezidi",
+            // new in ICU 70
+            "Cypro_Minoan", "Old_Uyghur", "Tangsa", "Toto", "Vithkuqi",
         };
         static const char* expectedShort[] = {
             "Bali", "Batk", "Blis", "Brah", "Cham", "Cirt", "Cyrs", "Egyd", "Egyh", "Egyp",
@@ -453,6 +457,10 @@ void TestUScriptCodeAPI(){
             "Dogr", "Gong", "Maka", "Medf", "Rohg", "Sogd", "Sogo",
             // new in ICU 64
             "Elym", "Hmnp", "Nand", "Wcho",
+            // new in ICU 66
+            "Chrs", "Diak", "Kits", "Yezi",
+            // new in ICU 70
+            "Cpmn", "Ougr", "Tnsa", "Toto", "Vith",
         };
         int32_t j = 0;
         if(UPRV_LENGTHOF(expectedLong)!=(USCRIPT_CODE_LIMIT-USCRIPT_BALINESE)) {
