@@ -45,6 +45,7 @@ static void TestOptions(void);
 static void TestGetFieldDisplayNames(void);
 static void TestGetDefaultHourCycle(void);
 static void TestGetDefaultHourCycleOnEmptyInstance(void);
+static void TestEras(void);
 
 void addDateTimePatternGeneratorTest(TestNode** root) {
     TESTCASE(TestOpenClose);
@@ -54,6 +55,7 @@ void addDateTimePatternGeneratorTest(TestNode** root) {
     TESTCASE(TestGetFieldDisplayNames);
     TESTCASE(TestGetDefaultHourCycle);
     TESTCASE(TestGetDefaultHourCycleOnEmptyInstance);
+    TESTCASE(TestEras);
 }
 
 /*
@@ -531,7 +533,7 @@ static void TestGetDefaultHourCycle() {
         { "fi",       UDAT_HOUR_CYCLE_23 },
         { "fr",       UDAT_HOUR_CYCLE_23 },
         { "ja_JP",    UDAT_HOUR_CYCLE_23 },
-        { "zh_CN",    UDAT_HOUR_CYCLE_12 },
+        { "zh_CN",    UDAT_HOUR_CYCLE_23 },
         { "zh_HK",    UDAT_HOUR_CYCLE_12 },
         { "zh_TW",    UDAT_HOUR_CYCLE_12 },
         { "ko_KR",    UDAT_HOUR_CYCLE_12 },
@@ -548,7 +550,7 @@ static void TestGetDefaultHourCycle() {
         } else {
             UDateFormatHourCycle actual = udatpg_getDefaultHourCycle(dtpgen, &status);
             if (U_FAILURE(status) || testDataPtr->expected != actual) {
-                log_err("ERROR dtpgen locale %s udatpg_getDefaultHourCycle expecte to get %d but get %d\n",
+                log_err("ERROR dtpgen locale %s udatpg_getDefaultHourCycle expected to get %d but get %d\n",
                         testDataPtr->locale, testDataPtr->expected, actual);
             }
             udatpg_close(dtpgen);
@@ -556,7 +558,7 @@ static void TestGetDefaultHourCycle() {
     }
 }
 
-// Ensure that calling udatpg_getDefaultHourCycle on an empty instance doesn't call UPRV_UNREACHABLE/abort.
+// Ensure that calling udatpg_getDefaultHourCycle on an empty instance doesn't call UPRV_UNREACHABLE_EXIT/abort.
 static void TestGetDefaultHourCycleOnEmptyInstance() {
     UErrorCode status = U_ZERO_ERROR;
     UDateTimePatternGenerator * dtpgen = udatpg_openEmpty(&status);
@@ -580,4 +582,36 @@ static void TestGetDefaultHourCycleOnEmptyInstance() {
     udatpg_close(dtpgen);
 }
 
+// Test for ICU-21202: Make sure DateTimePatternGenerator supplies an era field for year formats using the
+// Buddhist and Japanese calendars for all English-speaking locales.
+static void TestEras(void) {
+    const char* localeIDs[] = {
+        "en_US@calendar=japanese",
+        "en_GB@calendar=japanese",
+        "en_150@calendar=japanese",
+        "en_001@calendar=japanese",
+        "en@calendar=japanese",
+        "en_US@calendar=buddhist",
+        "en_GB@calendar=buddhist",
+        "en_150@calendar=buddhist",
+        "en_001@calendar=buddhist",
+        "en@calendar=buddhist",
+    };
+    
+    UErrorCode err = U_ZERO_ERROR;
+    for (int32_t i = 0; i < UPRV_LENGTHOF(localeIDs); i++) {
+        const char* locale = localeIDs[i];
+        UDateTimePatternGenerator* dtpg = udatpg_open(locale, &err);
+        if (U_SUCCESS(err)) {
+            UChar pattern[200];
+            udatpg_getBestPattern(dtpg, u"y", 1, pattern, 200, &err);
+            
+            if (u_strchr(pattern, u'G') == NULL) {
+                log_err("missing era field for locale %s\n", locale);
+            }
+        }
+        udatpg_close(dtpg);
+    }
+}
+
 #endif
