@@ -35,6 +35,7 @@
 #include "unicode/unistr.h"
 #include "unicode/parsepos.h"
 #include "unicode/uniset.h"
+#include "unicode/usetiter.h"
 #include "unicode/ustring.h"
 #include "unicode/ures.h"
 #include "unicode/utf16.h"
@@ -77,6 +78,7 @@ ConversionTest::runIndexedTest(int32_t index, UBool exec, const char *&name, cha
     TESTCASE_AUTO(TestGetUnicodeSet2);
     TESTCASE_AUTO(TestDefaultIgnorableCallback);
     TESTCASE_AUTO(TestUTF8ToUTF8Overflow);
+    TESTCASE_AUTO(TestUTF8ToUTF8Streaming);
     TESTCASE_AUTO_END;
 }
 
@@ -642,14 +644,21 @@ ConversionTest::TestGetUnicodeSet2() {
     delete [] s0;
 }
 
-// Test all codepoints which has the default ignorable Unicode property are ignored if they have no mapping
-// If there are any failures, the hard coded list (IS_DEFAULT_IGNORABLE_CODE_POINT) in ucnv_err.c should be updated
+// Test that all code points which have the default ignorable Unicode property
+// are ignored if they have no mapping.
+// If there are any failures, the hard coded list (IS_DEFAULT_IGNORABLE_CODE_POINT)
+// in ucnv_err.cpp should be updated.
 void
 ConversionTest::TestDefaultIgnorableCallback() {
     UErrorCode status = U_ZERO_ERROR;
     const char *cnv_name = "euc-jp-2007";
     const char *pattern_ignorable = "[:Default_Ignorable_Code_Point:]";
-    const char *pattern_not_ignorable = "[:^Default_Ignorable_Code_Point:]";
+    const char *pattern_not_ignorable =
+        "[[:^Default_Ignorable_Code_Point:]"
+        // For test performance, skip large ranges that will likely remain unassigned
+        // for a long time, and private use code points.
+        "-[\\U00040000-\\U000DFFFF]-[:Co:]"
+        "]";
 
     LocalPointer<UnicodeSet> set_ignorable(new UnicodeSet(pattern_ignorable, status));
     if (U_FAILURE(status)) {
@@ -677,12 +686,12 @@ ConversionTest::TestDefaultIgnorableCallback() {
     int32_t outputLength;
     
     // test default ignorables are ignored
-    int size = set_ignorable->size();
-    for (int i = 0; i < size; i++) {
+    UnicodeSetIterator iter(*set_ignorable);
+    while (iter.next()) {
         status = U_ZERO_ERROR;
         outputLength= 0;
 
-        input[0] = set_ignorable->charAt(i);
+        input[0] = iter.getCodepoint();
 
         outputLength = ucnv_fromUChars(cnv.getAlias(), output, 10, UnicodeString::fromUTF32(input, 1).getTerminatedBuffer(), -1, &status);
         if (U_FAILURE(status) || outputLength != 0) {
@@ -691,12 +700,12 @@ ConversionTest::TestDefaultIgnorableCallback() {
     }
 
     // test non-ignorables are not ignored
-    size = set_not_ignorable->size();
-    for (int i = 0; i < size; i++) {
+    iter.reset(*set_not_ignorable);
+    while (iter.next()) {
         status = U_ZERO_ERROR;
         outputLength= 0;
 
-        input[0] = set_not_ignorable->charAt(i);
+        input[0] = iter.getCodepoint();
 
         if (input[0] == 0) {
             continue;
@@ -830,6 +839,65 @@ ConversionTest::TestUTF8ToUTF8Overflow() {
     }
 }
 
+void
+ConversionTest::TestUTF8ToUTF8Streaming() {
+    IcuTestErrorCode errorCode(*this, "TestUTF8ToUTF8Streaming");
+    LocalUConverterPointer cnv1(ucnv_open("UTF-8", errorCode));
+    LocalUConverterPointer cnv2(ucnv_open("UTF-8", errorCode));
+
+    // UTF8 encoded cyrillic part of 'Lorem ipsum'
+    static const char* text =
+        "\xd0\xb5\xd1\x82\x20\xd1\x81\xd1\x86\xd0\xb0\xd0\xb5\xd0\xb2\xd0"
+        "\xbe\xd0\xbb\xd0\xb0\x20\xd1\x81\xd0\xb0\xd0\xb4\xd0\xb8\xd0\xbf"
+        "\xd1\x81\xd1\x86\xd0\xb8\xd0\xbd\xd0\xb3\x20\xd0\xb0\xd1\x86\xd1"
+        "\x86\xd0\xbe\xd0\xbc\xd0\xbc\xd0\xbe\xd0\xb4\xd0\xb0\xd1\x80\xd0"
+        "\xb5\x20\xd1\x85\xd0\xb0\xd1\x81";
+
+    int32_t chunk1 = 25; // partial lead at the end: 0xd0
+    int32_t chunk2 = 47; // partial tail at the beginning: 0xb0
+
+    char result[128];
+
+    int32_t sourceLen = (int32_t)strlen(text);
+    const char* source = text;
+    const char* sourceLimit = text + chunk1;
+
+    int32_t targetLen = sizeof(result);
+    char* target = result;
+    const char* targetLimit = result + targetLen;
+
+    UChar buffer16[20];
+    UChar* pivotSource = buffer16;
+    UChar* pivotTarget = buffer16;
+    const UChar* pivotLimit = buffer16 + UPRV_LENGTHOF(buffer16);
+
+    int32_t length;
+    ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
+        &target, result + targetLen, &source, sourceLimit,
+        buffer16, &pivotSource, &pivotTarget, pivotLimit,
+        FALSE, FALSE, errorCode);
+
+    length = (int32_t)(target - result);
+    targetLen -= length;
+    assertEquals("First chunk -1 doesn't match converted length", chunk1 - 1, length);
+
+    source = text + chunk1;
+    sourceLimit = source + chunk2;
+
+    // Convert the rest and flush.
+    ucnv_convertEx(cnv2.getAlias(), cnv1.getAlias(),
+        &target, targetLimit, &source, sourceLimit,
+        buffer16, &pivotSource, &pivotTarget, pivotLimit,
+        FALSE, TRUE, errorCode);
+
+    length = (int32_t)(target - result - length);
+    targetLen -= length;
+    assertEquals("Second chunk + 2 doesn't  match converted length", chunk2 + 1, length);
+
+    assertEquals("Full text length match", sourceLen, sizeof(result) - targetLen);
+    assertSuccess("UTF-8->UTF-8", errorCode);
+}
+
 // open testdata or ICU data converter ------------------------------------- ***
 
 UConverter *
@@ -1185,9 +1253,13 @@ ConversionTest::ToUnicodeCase(ConversionCase &cc, UConverterToUCallback callback
             cc.offsets=NULL;
         }
         else {
-            memset(resultOffsets, -1, UPRV_LENGTHOF(resultOffsets));
+            for (int32_t i = 0; i < UPRV_LENGTHOF(resultOffsets); i++) {
+                resultOffsets[i] = -1;
+            }
+        }
+        for (int32_t i = 0; i < UPRV_LENGTHOF(result); i++) {
+            result[i] = -1;
         }
-        memset(result, -1, UPRV_LENGTHOF(result));
         errorCode.reset();
         resultLength=stepToUnicode(cc, cnv.getAlias(),
                                 result, UPRV_LENGTHOF(result),
@@ -1615,8 +1687,12 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
     ok=TRUE;
     for(i=0; i<UPRV_LENGTHOF(steps) && ok; ++i) {
         step=steps[i].step;
-        memset(resultOffsets, -1, UPRV_LENGTHOF(resultOffsets));
-        memset(result, -1, UPRV_LENGTHOF(result));
+        for (int32_t i = 0; i < UPRV_LENGTHOF(resultOffsets); i++) {
+            resultOffsets[i] = -1;
+        }
+        for (int32_t i = 0; i < UPRV_LENGTHOF(result); i++) {
+            result[i] = -1;
+        }
         errorCode=U_ZERO_ERROR;
         resultLength=stepFromUnicode(cc, cnv,
                                 result, UPRV_LENGTHOF(result),
