@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2001, International Business Machines Corporation and
@@ -31,7 +33,7 @@ public:
     CollationIteratorTest();
     virtual ~CollationIteratorTest();
 
-    void runIndexedTest(int32_t index, UBool exec, const char* &name, char* par = NULL);
+    void runIndexedTest(int32_t index, UBool exec, const char* &name, char* par = NULL) override;
 
     /**
     * Test that results from CollationElementIterator.next is equivalent to
