@@ -1,8 +1,15 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2009, International Business Machines Corporation and
  * others. All Rights Reserved. 
  ********************************************************************/
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #ifndef _COLL
 #include "unicode/coll.h"
 #endif
@@ -28,7 +35,14 @@ LotusCollationKoreanTest::LotusCollationKoreanTest()
 {
     UErrorCode status = U_ZERO_ERROR;
     myCollation = Collator::createInstance("ko_kr", status);
-    myCollation->setDecomposition(Normalizer::DECOMP);
+    if(U_SUCCESS(status)) {
+      myCollation->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
+    } else {
+      errcheckln(status, "Couldn't instantiate the collator with %s", u_errorName(status));
+      delete myCollation;
+      myCollation = 0;
+    }
+
 }
 
 LotusCollationKoreanTest::~LotusCollationKoreanTest()
@@ -49,38 +63,28 @@ const Collator::EComparisonResult LotusCollationKoreanTest::results[] = {
     Collator::LESS
 };
 
-void LotusCollationKoreanTest::doTest( UnicodeString source, UnicodeString target, Collator::EComparisonResult result)
-{
-    Collator::EComparisonResult compareResult = myCollation->compare(source, target);
-    CollationKey sortKey1, sortKey2;
-    UErrorCode key1status = U_ZERO_ERROR, key2status = U_ZERO_ERROR; //nos
-    myCollation->getCollationKey(source, /*nos*/ sortKey1, key1status );
-    myCollation->getCollationKey(target, /*nos*/ sortKey2, key2status );
-    if (U_FAILURE(key1status) || U_FAILURE(key2status)) {
-        errln("SortKey generation Failed.\n");
-        return;
-    }
-    Collator::EComparisonResult keyResult = sortKey1.compareTo(sortKey2);
-    reportCResult( source, target, sortKey1, sortKey2, compareResult, keyResult, compareResult, result );
-}
-
 void LotusCollationKoreanTest::TestTertiary(/* char* par */)
 {
     int32_t i = 0;
     myCollation->setStrength(Collator::TERTIARY);
 
     for (i = 0; i < 1; i++) {
-        doTest(testSourceCases[i], testTargetCases[i], results[i]);
+        doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
 }
 
 void LotusCollationKoreanTest::runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par*/ )
 {
     if (exec) logln("TestSuite LotusCollationKoreanTest: ");
-    switch (index) {
-        case 0: name = "TestTertiary";  if (exec)   TestTertiary(/* par */); break;
-        default: name = ""; break;
+    if(myCollation) {
+      switch (index) {
+          case 0: name = "TestTertiary";  if (exec)   TestTertiary(/* par */); break;
+          default: name = ""; break;
+      }
+    } else {
+      dataerrln("Class collator not instantiated");
+      name = "";
     }
 }
 
-
+#endif /* #if !UCONFIG_NO_COLLATION */
