@@ -23,6 +23,35 @@
 
 #if !UCONFIG_NO_FORMATTING
 
+void ListFormatterTest::runIndexedTest(int32_t index, UBool exec,
+                                       const char* &name, char* /*par */) {
+    TESTCASE_AUTO_BEGIN;
+    TESTCASE_AUTO(TestRoot);
+    TESTCASE_AUTO(TestBogus);
+    TESTCASE_AUTO(TestEnglish);
+    TESTCASE_AUTO(TestEnglishUS);
+    TESTCASE_AUTO(TestRussian);
+    TESTCASE_AUTO(TestMalayalam);
+    TESTCASE_AUTO(TestZulu);
+    TESTCASE_AUTO(TestOutOfOrderPatterns);
+    TESTCASE_AUTO(Test9946);
+    TESTCASE_AUTO(TestEnglishGB);
+    TESTCASE_AUTO(TestNynorsk);
+    TESTCASE_AUTO(TestChineseTradHK);
+    TESTCASE_AUTO(TestFieldPositionIteratorWith1Item);
+    TESTCASE_AUTO(TestFieldPositionIteratorWith2Items);
+    TESTCASE_AUTO(TestFieldPositionIteratorWith2ItemsPatternShift);
+    TESTCASE_AUTO(TestFieldPositionIteratorWith3Items);
+    TESTCASE_AUTO(TestFieldPositionIteratorWith3ItemsPatternShift);
+    TESTCASE_AUTO(TestFormattedValue);
+    TESTCASE_AUTO(TestDifferentStyles);
+    TESTCASE_AUTO(TestCreateStyled);
+    TESTCASE_AUTO(TestContextual);
+    TESTCASE_AUTO(TestNextPosition);
+    TESTCASE_AUTO(TestInt32Overflow);
+    TESTCASE_AUTO_END;
+}
+
 namespace {
 const char* attrString(int32_t attrId) {
   switch (attrId) {
@@ -33,10 +62,14 @@ const char* attrString(int32_t attrId) {
 }
 }  // namespace
 
-void ListFormatterTest::ExpectPositions(FieldPositionIterator& iter,
-                                        int32_t *values, int32_t tupleCount) {
+void ListFormatterTest::ExpectPositions(
+        const FormattedList& iter,
+        int32_t *values,
+        int32_t tupleCount,
+        UErrorCode& status) {
     UBool found[10];
-    FieldPosition fp;
+    ConstrainedFieldPosition cfp;
+    cfp.constrainCategory(UFIELD_CATEGORY_LIST);
     if (tupleCount > 10) {
       assertTrue("internal error, tupleCount too large", FALSE);
     } else {
@@ -44,11 +77,11 @@ void ListFormatterTest::ExpectPositions(FieldPositionIterator& iter,
             found[i] = FALSE;
         }
     }
-    while (iter.next(fp)) {
+    while (iter.nextPosition(cfp, status)) {
         UBool ok = FALSE;
-        int32_t id = fp.getField();
-        int32_t start = fp.getBeginIndex();
-        int32_t limit = fp.getEndIndex();
+        int32_t id = cfp.getField();
+        int32_t start = cfp.getStart();
+        int32_t limit = cfp.getLimit();
         char buf[128];
         sprintf(buf, "%24s %3d %3d %3d", attrString(id), id, start, limit);
         logln(buf);
@@ -217,57 +250,36 @@ void ListFormatterTest::TestEnglishGB() {
     CheckFourCases("en_GB", one, two, three, four, results, "TestEnglishGB()");
 }
 
-void ListFormatterTest::TestFieldPositionIteratorWontCrash() {
-    IcuTestErrorCode errorCode(*this, "TestFieldPositionIteratorWontCrash()");
-    LocalPointer<ListFormatter> formatter(
-        ListFormatter::createInstance(Locale("en"), errorCode));
-    if (U_FAILURE(errorCode)) {
-        dataerrln(
-            "ListFormatter::createInstance(Locale(\"en\"), errorCode) failed in "
-            "TestFieldPositionIteratorWontCrash: %s",
-            u_errorName(errorCode));
-        return;
-    }
-    UnicodeString data[3] = {"a", "bbb", "cc"};
-    UnicodeString actualResult;
-     // make sure NULL as FieldPositionIterator won't caused crash.
-    formatter->format(data, 3, actualResult, nullptr, errorCode);
-    if (U_FAILURE(errorCode)) {
-        dataerrln(
-            "ListFormatter::format(data, 3, nullptr, errorCode) "
-            "failed in TestFieldPositionIteratorWontCrash: %s",
-            u_errorName(errorCode));
-        return;
-    }
-}
-
 void ListFormatterTest::RunTestFieldPositionIteratorWithFormatter(
         ListFormatter* formatter,
         UnicodeString data[], int32_t n, int32_t expected[], int32_t tupleCount,
-        UnicodeString& appendTo, const char16_t *expectedFormatted,
+        const char16_t *expectedFormatted,
         const char* testName) {
     IcuTestErrorCode errorCode(*this, testName);
-    FieldPositionIterator iter;
-    formatter->format(data, n, appendTo, &iter, errorCode);
+    FormattedList fl = formatter->formatStringsToValue(data, n, errorCode);
+    UnicodeString actual = fl.toString(errorCode);
     if (U_FAILURE(errorCode)) {
         dataerrln(
             "ListFormatter::format(data, %d, &iter, errorCode) "
             "failed in %s: %s", n, testName, u_errorName(errorCode));
         return;
     }
-    if (appendTo != expectedFormatted) {
-        errln(UnicodeString("Expected: |") + expectedFormatted +  "|, Actual: |" + appendTo + "|");
+    if (actual != expectedFormatted) {
+        errln(UnicodeString("Expected: |") + expectedFormatted +  "|, Actual: |" + actual + "|");
     }
-    ExpectPositions(iter, expected, tupleCount);
+    ExpectPositions(fl, expected, tupleCount, errorCode);
 }
 
 void ListFormatterTest::RunTestFieldPositionIteratorWithNItemsPatternShift(
         UnicodeString data[], int32_t n, int32_t expected[], int32_t tupleCount,
-        UnicodeString& appendTo, const char16_t *expectedFormatted,
+        const char16_t *expectedFormatted,
         const char* testName) {
     IcuTestErrorCode errorCode(*this, testName);
-    LocalPointer<ListFormatter> formatter(
-        ListFormatter::createInstance(Locale("ur", "IN"), "unit-narrow", errorCode));
+    LocalPointer<ListFormatter> formatter(ListFormatter::createInstance(
+        Locale("ur", "IN"),
+        ULISTFMT_TYPE_UNITS,
+        ULISTFMT_WIDTH_NARROW,
+        errorCode));
     if (U_FAILURE(errorCode)) {
         dataerrln(
             "ListFormatter::createInstance(Locale(\"ur\", \"IN\"), \"unit-narrow\", errorCode) failed in "
@@ -276,12 +288,12 @@ void ListFormatterTest::RunTestFieldPositionIteratorWithNItemsPatternShift(
     }
     RunTestFieldPositionIteratorWithFormatter(
         formatter.getAlias(),
-        data, n, expected, tupleCount, appendTo, expectedFormatted, testName);
+        data, n, expected, tupleCount, expectedFormatted, testName);
 }
 
 void ListFormatterTest::RunTestFieldPositionIteratorWithNItems(
         UnicodeString data[], int32_t n, int32_t expected[], int32_t tupleCount,
-        UnicodeString& appendTo, const char16_t *expectedFormatted,
+        const char16_t *expectedFormatted,
         const char* testName) {
     IcuTestErrorCode errorCode(*this, testName);
     LocalPointer<ListFormatter> formatter(
@@ -294,27 +306,7 @@ void ListFormatterTest::RunTestFieldPositionIteratorWithNItems(
     }
     RunTestFieldPositionIteratorWithFormatter(
         formatter.getAlias(),
-        data, n, expected, tupleCount, appendTo, expectedFormatted, testName);
-}
-
-void ListFormatterTest::TestFieldPositionIteratorWith3ItemsAndDataBefore() {
-    //  0         1         2
-    //  0123456789012345678901234567
-    // "Hello World: a, bbb, and cc"
-    UnicodeString data[3] = {"a", "bbb", "cc"};
-    int32_t expected[] = {
-        ULISTFMT_ELEMENT_FIELD, 13, 14,
-        ULISTFMT_LITERAL_FIELD, 14, 16,
-        ULISTFMT_ELEMENT_FIELD, 16, 19,
-        ULISTFMT_LITERAL_FIELD, 19, 25,
-        ULISTFMT_ELEMENT_FIELD, 25, 27
-    };
-    int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo(u"Hello World: ");
-    RunTestFieldPositionIteratorWithNItems(
-        data, 3, expected, tupleCount, appendTo,
-        u"Hello World: a, bbb, and cc",
-        "TestFieldPositionIteratorWith3ItemsAndDataBefore");
+        data, n, expected, tupleCount, expectedFormatted, testName);
 }
 
 void ListFormatterTest::TestFieldPositionIteratorWith3Items() {
@@ -330,9 +322,8 @@ void ListFormatterTest::TestFieldPositionIteratorWith3Items() {
         ULISTFMT_ELEMENT_FIELD, 12, 14
     };
     int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo;
     RunTestFieldPositionIteratorWithNItems(
-        data, 3, expected, tupleCount, appendTo,
+        data, 3, expected, tupleCount,
         u"a, bbb, and cc",
         "TestFieldPositionIteratorWith3Items");
 }
@@ -350,31 +341,12 @@ void ListFormatterTest::TestFieldPositionIteratorWith3ItemsPatternShift() {
         ULISTFMT_ELEMENT_FIELD, 0, 2
     };
     int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo;
     RunTestFieldPositionIteratorWithNItemsPatternShift(
-        data, 3, expected, tupleCount, appendTo,
+        data, 3, expected, tupleCount,
         u"cc bbb a",
         "TestFieldPositionIteratorWith3ItemsPatternShift");
 }
 
-void ListFormatterTest::TestFieldPositionIteratorWith2ItemsAndDataBefore() {
-    //  0         1
-    //  0123456789012345
-    // "Foo: bbb and cc"
-    UnicodeString data[2] = {"bbb", "cc"};
-    int32_t expected[] = {
-        ULISTFMT_ELEMENT_FIELD, 5, 8,
-        ULISTFMT_LITERAL_FIELD, 8, 13,
-        ULISTFMT_ELEMENT_FIELD, 13, 15
-    };
-    int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo("Foo: ");
-    RunTestFieldPositionIteratorWithNItems(
-        data, 2, expected, tupleCount, appendTo,
-        u"Foo: bbb and cc",
-        "TestFieldPositionIteratorWith2ItemsAndDataBefore");
-}
-
 void ListFormatterTest::TestFieldPositionIteratorWith2Items() {
     //  0         1
     //  01234567890
@@ -386,9 +358,8 @@ void ListFormatterTest::TestFieldPositionIteratorWith2Items() {
         ULISTFMT_ELEMENT_FIELD, 8, 10
     };
     int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo;
     RunTestFieldPositionIteratorWithNItems(
-        data, 2, expected, tupleCount, appendTo,
+        data, 2, expected, tupleCount,
         u"bbb and cc",
         "TestFieldPositionIteratorWith2Items");
 }
@@ -404,28 +375,12 @@ void ListFormatterTest::TestFieldPositionIteratorWith2ItemsPatternShift() {
         ULISTFMT_ELEMENT_FIELD, 0, 2
     };
     int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo;
     RunTestFieldPositionIteratorWithNItemsPatternShift(
-        data, 2, expected, tupleCount, appendTo,
+        data, 2, expected, tupleCount,
         u"cc bbb",
         "TestFieldPositionIteratorWith2ItemsPatternShift");
 }
 
-void ListFormatterTest::TestFieldPositionIteratorWith1ItemAndDataBefore() {
-    //  012345678
-    // "Hello cc"
-    UnicodeString data[1] = {"cc"};
-    int32_t expected[] = {
-        ULISTFMT_ELEMENT_FIELD, 6, 8
-    };
-    int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo("Hello ");
-    RunTestFieldPositionIteratorWithNItems(
-        data, 1, expected, tupleCount, appendTo,
-        u"Hello cc",
-        "TestFieldPositionIteratorWith1ItemAndDataBefore");
-}
-
 void ListFormatterTest::TestFieldPositionIteratorWith1Item() {
     //  012
     // "cc"
@@ -434,9 +389,8 @@ void ListFormatterTest::TestFieldPositionIteratorWith1Item() {
         ULISTFMT_ELEMENT_FIELD, 0, 2
     };
     int32_t tupleCount = sizeof(expected)/(3 * sizeof(*expected));
-    UnicodeString appendTo;
     RunTestFieldPositionIteratorWithNItems(
-        data, 1, expected, tupleCount, appendTo,
+        data, 1, expected, tupleCount,
         u"cc",
         "TestFieldPositionIteratorWith1Item");
 }
@@ -524,8 +478,9 @@ void ListFormatterTest::TestOutOfOrderPatterns() {
     };
 
     IcuTestErrorCode errorCode(*this, "TestOutOfOrderPatterns()");
+    Locale locale("en");
     ListFormatData data("{1} after {0}", "{1} after the first {0}",
-                        "{1} after {0}", "{1} in the last after {0}");
+                        "{1} after {0}", "{1} in the last after {0}", locale);
     ListFormatter formatter(data, errorCode);
 
     UnicodeString input1[] = {one};
@@ -543,10 +498,10 @@ void ListFormatterTest::TestOutOfOrderPatterns() {
 
 void ListFormatterTest::TestFormattedValue() {
     IcuTestErrorCode status(*this, "TestFormattedValue");
-    LocalPointer<ListFormatter> fmt(ListFormatter::createInstance("en", status));
-    if (status.errIfFailureAndReset()) { return; }
 
     {
+        LocalPointer<ListFormatter> fmt(ListFormatter::createInstance("en", status));
+        if (status.errIfFailureAndReset()) { return; }
         const char16_t* message = u"Field position test 1";
         const char16_t* expectedString = u"hello, wonderful, and world";
         const UnicodeString inputs[] = {
@@ -572,18 +527,105 @@ void ListFormatterTest::TestFormattedValue() {
             expectedFieldPositions,
             UPRV_LENGTHOF(expectedFieldPositions));
     }
+
+    {
+        LocalPointer<ListFormatter> fmt(ListFormatter::createInstance("zh", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_SHORT, status));
+        if (status.errIfFailureAndReset()) { return; }
+        const char16_t* message = u"Field position test 2 (ICU-21340)";
+        const char16_t* expectedString = u"aabbbbbbbccc";
+        const UnicodeString inputs[] = {
+            u"aa",
+            u"bbbbbbb",
+            u"ccc"
+        };
+        FormattedList result = fmt->formatStringsToValue(inputs, UPRV_LENGTHOF(inputs), status);
+        static const UFieldPositionWithCategory expectedFieldPositions[] = {
+            // field, begin index, end index
+            {UFIELD_CATEGORY_LIST_SPAN, 0, 0, 2},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 0, 2},
+            {UFIELD_CATEGORY_LIST_SPAN, 1, 2, 9},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 2, 9},
+            {UFIELD_CATEGORY_LIST_SPAN, 2, 9, 12},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 9, 12}};
+        checkMixedFormattedValue(
+            message,
+            result,
+            expectedString,
+            expectedFieldPositions,
+            UPRV_LENGTHOF(expectedFieldPositions));
+    }
+
+    {
+        LocalPointer<ListFormatter> fmt(ListFormatter::createInstance("en", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_SHORT, status));
+        if (status.errIfFailureAndReset()) { return; }
+        const char16_t* message = u"ICU-21383 Long list";
+        const char16_t* expectedString = u"a, b, c, d, e, f, g, h, i";
+        const UnicodeString inputs[] = {
+            u"a",
+            u"b",
+            u"c",
+            u"d",
+            u"e",
+            u"f",
+            u"g",
+            u"h",
+            u"i",
+        };
+        FormattedList result = fmt->formatStringsToValue(inputs, UPRV_LENGTHOF(inputs), status);
+        static const UFieldPositionWithCategory expectedFieldPositions[] = {
+            // field, begin index, end index
+            {UFIELD_CATEGORY_LIST_SPAN, 0, 0, 1},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 0, 1},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 1, 3},
+            {UFIELD_CATEGORY_LIST_SPAN, 1, 3, 4},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 3, 4},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 4, 6},
+            {UFIELD_CATEGORY_LIST_SPAN, 2, 6, 7},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 6, 7},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 7, 9},
+            {UFIELD_CATEGORY_LIST_SPAN, 3, 9, 10},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 9, 10},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 10, 12},
+            {UFIELD_CATEGORY_LIST_SPAN, 4, 12, 13},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 12, 13},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 13, 15},
+            {UFIELD_CATEGORY_LIST_SPAN, 5, 15, 16},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 15, 16},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 16, 18},
+            {UFIELD_CATEGORY_LIST_SPAN, 6, 18, 19},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 18, 19},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 19, 21},
+            {UFIELD_CATEGORY_LIST_SPAN, 7, 21, 22},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 21, 22},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_LITERAL_FIELD, 22, 24},
+            {UFIELD_CATEGORY_LIST_SPAN, 8, 24, 25},
+            {UFIELD_CATEGORY_LIST, ULISTFMT_ELEMENT_FIELD, 24, 25},
+            };
+        checkMixedFormattedValue(
+            message,
+            result,
+            expectedString,
+            expectedFieldPositions,
+            UPRV_LENGTHOF(expectedFieldPositions));
+    }
 }
 
-void ListFormatterTest::DoTheRealListStyleTesting(Locale locale,
-        UnicodeString items[], int itemCount,
-        const char* style, const char* expected, IcuTestErrorCode status) {
+void ListFormatterTest::DoTheRealListStyleTesting(
+        Locale locale,
+        UnicodeString items[],
+        int itemCount,
+        UListFormatterType type,
+        UListFormatterWidth width,
+        const char* expected,
+        IcuTestErrorCode status) {
 
     LocalPointer<ListFormatter> formatter(
-            ListFormatter::createInstance(locale, style, status));
+            ListFormatter::createInstance(locale, type, width, status));
 
     UnicodeString actualResult;
     formatter->format(items, itemCount, actualResult, status);
-    assertEquals(style, UnicodeString(expected), actualResult);
+    assertEquals(Int64ToUnicodeString(type) + "-" + Int64ToUnicodeString(width),
+        UnicodeString(expected), actualResult);
 }
 
 void ListFormatterTest::TestDifferentStyles() {
@@ -591,79 +633,211 @@ void ListFormatterTest::TestDifferentStyles() {
     UnicodeString input[4] = { u"rouge", u"jaune", u"bleu", u"vert" };
     IcuTestErrorCode status(*this, "TestDifferentStyles()");
 
-    DoTheRealListStyleTesting(locale, input, 4, "standard", "rouge, jaune, bleu et vert", status);
-    DoTheRealListStyleTesting(locale, input, 4, "or", "rouge, jaune, bleu ou vert", status);
-    DoTheRealListStyleTesting(locale, input, 4, "unit", "rouge, jaune, bleu et vert", status);
-    DoTheRealListStyleTesting(locale, input, 4, "unit-narrow", "rouge jaune bleu vert", status);
-    DoTheRealListStyleTesting(locale, input, 4, "unit-short", "rouge, jaune, bleu et vert", status);
+    DoTheRealListStyleTesting(locale, input, 4, ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_WIDE, "rouge, jaune, bleu et vert", status);
+    DoTheRealListStyleTesting(locale, input, 4, ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_WIDE, "rouge, jaune, bleu ou vert", status);
+    DoTheRealListStyleTesting(locale, input, 4, ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_WIDE, "rouge, jaune, bleu et vert", status);
+    DoTheRealListStyleTesting(locale, input, 4, ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_NARROW, "rouge jaune bleu vert", status);
+    DoTheRealListStyleTesting(locale, input, 4, ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_SHORT, "rouge, jaune, bleu et vert", status);
 }
 
-void ListFormatterTest::TestBadStylesFail() {
-    Locale locale("fr");
-    const char * badStyles[4] = { "", "duration", "duration-short", "something-clearly-wrong" };
-    IcuTestErrorCode status(*this, "TestBadStylesFail()");
-
-    for (int i = 0; i < 4; ++i) {
-      LocalPointer<ListFormatter> formatter(ListFormatter::createInstance(locale, badStyles[i], status));
-      if (!status.expectErrorAndReset(U_MISSING_RESOURCE_ERROR, "style \"%s\"", badStyles[i])) {
-        // Do nothing, expectErrorAndReset already reports the error
-      }
+void ListFormatterTest::TestCreateStyled() {
+    IcuTestErrorCode status(*this, "TestCreateStyled");
+    // Locale en has interesting data
+    struct TestCase {
+        const char* locale;
+        UListFormatterType type;
+        UListFormatterWidth width;
+        const char16_t* expected3;
+        const char16_t* expected2;
+        const char16_t* expected1;
+    } cases[] = {
+        { "pt", ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_WIDE, u"A, B e C", u"A e B", u"A" },
+        { "pt", ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_SHORT, u"A, B e C", u"A e B", u"A" },
+        { "pt", ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_NARROW, u"A, B, C", u"A, B", u"A" },
+        { "pt", ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_WIDE, u"A, B ou C", u"A ou B", u"A" },
+        { "pt", ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_SHORT, u"A, B ou C", u"A ou B", u"A" },
+        { "pt", ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_NARROW, u"A, B ou C", u"A ou B", u"A" },
+        { "pt", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_WIDE, u"A, B e C", u"A e B", u"A" },
+        { "pt", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_SHORT, u"A, B e C", u"A e B", u"A" },
+        { "pt", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_NARROW, u"A B C", u"A B", u"A" },
+        { "en", ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_WIDE, u"A, B, and C", u"A and B", u"A" },
+        { "en", ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_SHORT, u"A, B, & C", u"A & B", u"A" },
+        { "en", ULISTFMT_TYPE_AND, ULISTFMT_WIDTH_NARROW, u"A, B, C", u"A, B", u"A" },
+        { "en", ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_WIDE, u"A, B, or C", u"A or B", u"A" },
+        { "en", ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_SHORT, u"A, B, or C", u"A or B", u"A" },
+        { "en", ULISTFMT_TYPE_OR, ULISTFMT_WIDTH_NARROW, u"A, B, or C", u"A or B", u"A" },
+        { "en", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_WIDE, u"A, B, C", u"A, B", u"A" },
+        { "en", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_SHORT, u"A, B, C", u"A, B", u"A" },
+        { "en", ULISTFMT_TYPE_UNITS, ULISTFMT_WIDTH_NARROW, u"A B C", u"A B", u"A" },
+    };
+    for (auto cas : cases) {
+        LocalPointer<ListFormatter> fmt(
+            ListFormatter::createInstance(cas.locale, cas.type, cas.width, status),
+            status);
+        if (status.errIfFailureAndReset()) {
+            continue;
+        }
+        UnicodeString message = UnicodeString(u"TestCreateStyled loc=")
+            + cas.locale + u" type="
+            + Int64ToUnicodeString(cas.type) + u" width="
+            + Int64ToUnicodeString(cas.width);
+        const UnicodeString inputs3[] = {
+            u"A",
+            u"B",
+            u"C"
+        };
+        FormattedList result = fmt->formatStringsToValue(inputs3, UPRV_LENGTHOF(inputs3), status);
+        assertEquals(message, cas.expected3, result.toTempString(status));
+        const UnicodeString inputs2[] = {
+            u"A",
+            u"B"
+        };
+        result = fmt->formatStringsToValue(inputs2, UPRV_LENGTHOF(inputs2), status);
+        assertEquals(message, cas.expected2, result.toTempString(status));
+        const UnicodeString inputs1[] = {
+            u"A"
+        };
+        result = fmt->formatStringsToValue(inputs1, UPRV_LENGTHOF(inputs1), status);
+        assertEquals(message, cas.expected1, result.toTempString(status));
     }
 }
 
-void ListFormatterTest::runIndexedTest(int32_t index, UBool exec,
-                                       const char* &name, char* /*par */) {
-    switch(index) {
-        case 0: name = "TestRoot"; if (exec) TestRoot(); break;
-        case 1: name = "TestBogus"; if (exec) TestBogus(); break;
-        case 2: name = "TestEnglish"; if (exec) TestEnglish(); break;
-        case 3: name = "TestEnglishUS"; if (exec) TestEnglishUS(); break;
-        case 4: name = "TestRussian"; if (exec) TestRussian(); break;
-        case 5: name = "TestMalayalam"; if (exec) TestMalayalam(); break;
-        case 6: name = "TestZulu"; if (exec) TestZulu(); break;
-        case 7: name = "TestOutOfOrderPatterns"; if (exec) TestOutOfOrderPatterns(); break;
-        case 8: name = "Test9946"; if (exec) Test9946(); break;
-        case 9: name = "TestEnglishGB"; if (exec) TestEnglishGB(); break;
-        case 10: name = "TestNynorsk"; if (exec) TestNynorsk(); break;
-        case 11: name = "TestChineseTradHK"; if (exec) TestChineseTradHK(); break;
-        case 12: name = "TestFieldPositionIteratorWontCrash";
-                 if (exec) TestFieldPositionIteratorWontCrash();
-                 break;
-        case 13: name = "TestFieldPositionIteratorWith1Item";
-                 if (exec) TestFieldPositionIteratorWith1Item();
-                 break;
-        case 14: name = "TestFieldPositionIteratorWith1ItemAndDataBefore";
-                 if (exec) TestFieldPositionIteratorWith1ItemAndDataBefore();
-                 break;
-        case 15: name = "TestFieldPositionIteratorWith2Items";
-                 if (exec) TestFieldPositionIteratorWith2Items();
-                 break;
-        case 16: name = "TestFieldPositionIteratorWith2ItemsAndDataBefore";
-                 if (exec) TestFieldPositionIteratorWith2ItemsAndDataBefore();
-                 break;
-        case 17: name = "TestFieldPositionIteratorWith2ItemsPatternShift";
-                 if (exec) TestFieldPositionIteratorWith2ItemsPatternShift();
-                 break;
-        case 18: name = "TestFieldPositionIteratorWith3Items";
-                 if (exec) TestFieldPositionIteratorWith3Items();
-                 break;
-        case 19: name = "TestFieldPositionIteratorWith3ItemsAndDataBefore";
-                 if (exec) TestFieldPositionIteratorWith3ItemsAndDataBefore();
-                 break;
-        case 20: name = "TestFieldPositionIteratorWith3ItemsPatternShift";
-                 if (exec) TestFieldPositionIteratorWith3ItemsPatternShift();
-                 break;
-        case 21: name = "TestFormattedValue";
-                 if (exec) TestFormattedValue();
-                 break;
-        case 22: name = "TestDifferentStyles";
-                 if (exec) TestDifferentStyles();
-                 break;
-        case 23: name = "TestBadStylesFail";
-                 if (exec) TestBadStylesFail();
-                 break;
-        default: name = ""; break;
+void ListFormatterTest::TestContextual() {
+    IcuTestErrorCode status(*this, "TestContextual");
+    std::vector<std::string> es = { "es", "es_419" , "es_PY", "es_DO" };
+    std::vector<std::string> he = { "he", "he_IL", "iw", "iw_IL" };
+    UListFormatterWidth widths [] = {
+        ULISTFMT_WIDTH_WIDE, ULISTFMT_WIDTH_SHORT, ULISTFMT_WIDTH_NARROW
+    };
+    struct TestCase {
+        std::vector<std::string> locales;
+        UListFormatterType type;
+        const char16_t* expected;
+        const char16_t* data1;
+        const char16_t* data2;
+        const char16_t* data3;
+    } cases[] = {
+        { es, ULISTFMT_TYPE_AND, u"fascinante e increíblemente",
+          u"fascinante",                     u"increíblemente",       nullptr },
+        { es, ULISTFMT_TYPE_AND, u"Comunicaciones Industriales e IIoT",
+          u"Comunicaciones Industriales",    u"IIoT",                 nullptr },
+        { es, ULISTFMT_TYPE_AND, u"España e Italia",         u"España",   u"Italia",      nullptr },
+        { es, ULISTFMT_TYPE_AND, u"hijas intrépidas e hijos solidarios",
+          u"hijas intrépidas",               u"hijos solidarios",     nullptr },
+        { es, ULISTFMT_TYPE_AND, u"a un hombre e hirieron a otro",
+          u"a un hombre",                    u"hirieron a otro",      nullptr },
+        { es, ULISTFMT_TYPE_AND, u"hija e hijo",             u"hija",     u"hijo",        nullptr },
+        { es, ULISTFMT_TYPE_AND, u"esposa, hija e hijo",     u"esposa",   u"hija",        u"hijo" },
+        // For 'y' exception
+        { es, ULISTFMT_TYPE_AND, u"oro y hierro",            u"oro",      u"hierro",      nullptr },
+        { es, ULISTFMT_TYPE_AND, u"agua y hielo",            u"agua",     u"hielo",       nullptr },
+        { es, ULISTFMT_TYPE_AND, u"colágeno y hialurónico",  u"colágeno", u"hialurónico", nullptr },
+
+        { es, ULISTFMT_TYPE_OR, u"desierto u oasis",         u"desierto", u"oasis",       nullptr },
+        { es, ULISTFMT_TYPE_OR, u"oasis, desierto u océano", u"oasis",    u"desierto",    u"océano" },
+        { es, ULISTFMT_TYPE_OR, u"7 u 8",                    u"7",        u"8",           nullptr },
+        { es, ULISTFMT_TYPE_OR, u"7 u 80",                   u"7",        u"80",          nullptr },
+        { es, ULISTFMT_TYPE_OR, u"7 u 800",                  u"7",        u"800",         nullptr },
+        { es, ULISTFMT_TYPE_OR, u"6, 7 u 8",                 u"6",        u"7",           u"8" },
+        { es, ULISTFMT_TYPE_OR, u"10 u 11",                  u"10",       u"11",          nullptr },
+        { es, ULISTFMT_TYPE_OR, u"10 o 111",                 u"10",       u"111",         nullptr },
+        { es, ULISTFMT_TYPE_OR, u"10 o 11.2",                u"10",       u"11.2",        nullptr },
+        { es, ULISTFMT_TYPE_OR, u"9, 10 u 11",               u"9",        u"10",          u"11" },
+
+        { he, ULISTFMT_TYPE_AND, u"a, b ו-c",               u"a",      u"b",      u"c" },
+        { he, ULISTFMT_TYPE_AND, u"a ו-b",                  u"a",      u"b",      nullptr },
+        { he, ULISTFMT_TYPE_AND, u"1, 2 ו-3",               u"1",      u"2",      u"3" },
+        { he, ULISTFMT_TYPE_AND, u"1 ו-2",                  u"1",      u"2",      nullptr },
+        { he, ULISTFMT_TYPE_AND, u"אהבה ומקווה",            u"אהבה",   u"מקווה",  nullptr },
+        { he, ULISTFMT_TYPE_AND, u"אהבה, מקווה ואמונה",     u"אהבה",   u"מקווה",  u"אמונה" },
+    };
+    for (auto width : widths) {
+        for (auto cas : cases) {
+            for (auto locale : cas.locales) {
+                LocalPointer<ListFormatter> fmt(
+                    ListFormatter::createInstance(locale.c_str(), cas.type, width, status),
+                    status);
+                if (status.errIfFailureAndReset()) {
+                    continue;
+                }
+                UnicodeString message = UnicodeString(u"TestContextual loc=")
+                    + locale.c_str() + u" type="
+                    + Int64ToUnicodeString(cas.type) + u" width="
+                    + Int64ToUnicodeString(width);
+                if (cas.data3 == nullptr) {
+                    const UnicodeString inputs2[] = { cas.data1, cas.data2 };
+                    FormattedList result = fmt->formatStringsToValue(inputs2, UPRV_LENGTHOF(inputs2), status);
+                    assertEquals(message, cas.expected, result.toTempString(status));
+                } else {
+                    const UnicodeString inputs3[] = { cas.data1, cas.data2, cas.data3 };
+                    FormattedList result = fmt->formatStringsToValue(inputs3, UPRV_LENGTHOF(inputs3), status);
+                    assertEquals(message, cas.expected, result.toTempString(status));
+                }
+            }
+        }
     }
 }
 
+void ListFormatterTest::TestNextPosition() {
+    IcuTestErrorCode status(*this, "TestNextPosition");
+    std::vector<std::string> locales = { "en", "es", "zh", "ja" };
+    UListFormatterWidth widths [] = {
+        ULISTFMT_WIDTH_WIDE, ULISTFMT_WIDTH_SHORT, ULISTFMT_WIDTH_NARROW
+    };
+    const char* widthStr [] = {"wide", "short", "narrow"};
+    UListFormatterType types [] = {
+        ULISTFMT_TYPE_AND, ULISTFMT_TYPE_OR, ULISTFMT_TYPE_UNITS
+    };
+    const char* typeStr [] = {"and", "or", "units"};
+    const UnicodeString inputs[] = { u"A1", u"B2", u"C3", u"D4" };
+    for (auto width : widths) {
+        for (auto type : types) {
+            for (auto locale : locales) {
+                LocalPointer<ListFormatter> fmt(
+                        ListFormatter::createInstance(locale.c_str(), type, width, status),
+                    status);
+                if (status.errIfFailureAndReset()) {
+                    continue;
+                }
+                for (int32_t n = 1; n <= UPRV_LENGTHOF(inputs); n++) {
+                    FormattedList result = fmt->formatStringsToValue(
+                        inputs, n, status);
+                    int32_t elements = 0;
+                    icu::ConstrainedFieldPosition cfpos;
+                    cfpos.constrainCategory(UFIELD_CATEGORY_LIST);
+                    while (result.nextPosition(cfpos, status) && U_SUCCESS(status)) {
+                        if (cfpos.getField() == ULISTFMT_ELEMENT_FIELD) {
+                            elements++;
+                        }
+                    }
+                    std::string msg = locale;
+                    // Test that if there are n elements (n=1..4) in the input, then the
+                    // nextPosition() should iterate through exactly n times
+                    // with field == ULISTFMT_ELEMENT_FIELD.
+                    assertEquals((msg
+                                  .append(" w=").append(widthStr[width])
+                                  .append(" t=").append(typeStr[type])).c_str(),
+                                 n, elements);
+                }
+            }
+        }
+    }
+}
+
+void ListFormatterTest::TestInt32Overflow() {
+    if (quick) {
+        return;
+    }
+    IcuTestErrorCode status(*this, "TestInt32Overflow");
+    LocalPointer<ListFormatter> fmt(ListFormatter::createInstance("en", status), status);
+    std::vector<UnicodeString> inputs;
+    UnicodeString input(0xAAAFF00, 0x00000042, 0xAAAFF00);
+    for (int32_t i = 0; i < 16; i++) {
+      inputs.push_back(input);
+    }
+    FormattedList result = fmt->formatStringsToValue(
+        inputs.data(), static_cast<int32_t>(inputs.size()), status);
+    status.expectErrorAndReset(U_INPUT_TOO_LONG_ERROR);
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
