@@ -31,7 +31,7 @@ class ListFormatterTest : public IntlTestWithFieldPosition {
     ListFormatterTest();
     virtual ~ListFormatterTest() {}
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0) override;
 
     void TestRoot();
     void TestBogus();
@@ -52,9 +52,10 @@ class ListFormatterTest : public IntlTestWithFieldPosition {
     void TestFieldPositionIteratorWith3ItemsPatternShift();
     void TestFormattedValue();
     void TestDifferentStyles();
-    void TestBadStylesFail();
     void TestCreateStyled();
     void TestContextual();
+    void TestNextPosition();
+    void TestInt32Overflow();
 
   private:
     void CheckFormatting(
@@ -109,8 +110,9 @@ class ListFormatterTest : public IntlTestWithFieldPosition {
     void DoTheRealListStyleTesting(
         Locale locale,
         UnicodeString items[],
-        int32_t itemCount,
-        const char* style,
+        int itemCount,
+        UListFormatterType type,
+        UListFormatterWidth width,
         const char* expected,
         IcuTestErrorCode status);
 
