@@ -24,7 +24,7 @@ public:
     MultithreadTest();
     virtual ~MultithreadTest();
     
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ) override;
 
     /**
      * test that threads even work
@@ -36,10 +36,6 @@ public:
      **/
     void TestArabicShapingThreads(void);
 	
-    /**
-     * test that mutexes work 
-     **/
-    void TestMutex(void);
 #if !UCONFIG_NO_FORMATTING
     /**
      * test that intl functions work in a multithreaded context
@@ -49,10 +45,10 @@ public:
     void TestCollators(void);
     void TestString();
     void TestAnyTranslit();
-    void TestConditionVariables();
     void TestUnifiedCache();
     void TestBreakTranslit();
-
+    void TestIncDec();
+    void Test20104();
 };
 
 #endif
