@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-* Copyright (C) 2007-2014, International Business Machines Corporation and    *
+* Copyright (C) 2007-2015, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 */
@@ -15,8 +17,9 @@
 #include "intltest.h"
 
 class TimeZoneFormatTest : public IntlTest {
+  public:
     // IntlTest override
-    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par);
+    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par) override;
 
     void TestTimeZoneRoundTrip(void);
     void TestTimeRoundTrip(void);
@@ -24,6 +27,13 @@ class TimeZoneFormatTest : public IntlTest {
     void TestISOFormat(void);
     void TestFormat(void);
     void TestFormatTZDBNames(void);
+    void TestFormatCustomZone(void);
+    void TestFormatTZDBNamesAllZoneCoverage(void);
+    void TestAdoptDefaultThreadSafe(void);
+    void TestCentralTime(void);
+
+    void RunTimeRoundTripTests(int32_t threadNumber);
+    void RunAdoptDefaultThreadSafeTests(int32_t threadNumber);
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
