@@ -12,6 +12,7 @@
 #include "unicode/simpletz.h"
 #include "unicode/smpdtfmt.h"
 #include "unicode/strenum.h"
+#include "unicode/gregocal.h"
 #include "tzregts.h"
 #include "calregts.h"
 #include "cmemory.h"
@@ -46,6 +47,7 @@ TimeZoneRegressionTest::runIndexedTest( int32_t index, UBool exec, const char* &
         CASE(16, TestJDK12API);
         CASE(17, Test4176686);
         CASE(18, Test4184229);
+        CASE(19, TestNegativeDaylightSaving);
         default: name = ""; break;
     }
 }
@@ -127,15 +129,21 @@ void TimeZoneRegressionTest:: Test4073215()
 {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString str, str2;
-    SimpleTimeZone *z = new SimpleTimeZone(0, "GMT");
-    if (z->useDaylightTime())
+    LocalPointer<SimpleTimeZone> z(new SimpleTimeZone(0, "GMT"), status);
+    if (U_FAILURE(status)) {
+        errln("Fail: Failed to create SimpleTimeZone %s", u_errorName(status));
+        return;
+    }
+    if (z->useDaylightTime()) {
         errln("Fail: Fix test to start with non-DST zone");
+    }
     z->setStartRule(UCAL_FEBRUARY, 1, UCAL_SUNDAY, 0, status);
     failure(status, "z->setStartRule()");
     z->setEndRule(UCAL_MARCH, -1, UCAL_SUNDAY, 0, status);
     failure(status, "z->setStartRule()");
-    if (!z->useDaylightTime())
+    if (!z->useDaylightTime()) {
         errln("Fail: DST not active");
+    }
 
     GregorianCalendar cal(1997, UCAL_JANUARY, 31, status);
     if(U_FAILURE(status)) {
@@ -143,7 +151,7 @@ void TimeZoneRegressionTest:: Test4073215()
       return;
     }
     failure(status, "new GregorianCalendar");
-    cal.adoptTimeZone(z);
+    cal.adoptTimeZone(z.orphan());
 
     SimpleDateFormat sdf((UnicodeString)"E d MMM yyyy G HH:mm", status); 
     if(U_FAILURE(status)) {
@@ -154,14 +162,14 @@ void TimeZoneRegressionTest:: Test4073215()
     failure(status, "new SimpleDateFormat");
 
     UDate jan31, mar1, mar31;
-
-    UBool indt = z->inDaylightTime(jan31=cal.getTime(status), status);
+    
+    UBool indt = cal.getTimeZone().inDaylightTime(jan31 = cal.getTime(status), status);
     failure(status, "inDaylightTime or getTime call on Jan 31");
     if (indt) {
         errln("Fail: Jan 31 inDaylightTime=TRUE, exp FALSE");
     }
     cal.set(1997, UCAL_MARCH, 1);
-    indt = z->inDaylightTime(mar1=cal.getTime(status), status);
+    indt = cal.getTimeZone().inDaylightTime(mar1 = cal.getTime(status), status);
     failure(status, "inDaylightTime or getTime call on Mar 1");
     if (!indt) {
         UnicodeString str;
@@ -170,7 +178,7 @@ void TimeZoneRegressionTest:: Test4073215()
         errln((UnicodeString)"Fail: " + str + " inDaylightTime=FALSE, exp TRUE");
     }
     cal.set(1997, UCAL_MARCH, 31);
-    indt = z->inDaylightTime(mar31=cal.getTime(status), status);
+    indt = cal.getTimeZone().inDaylightTime(mar31 = cal.getTime(status), status);
     failure(status, "inDaylightTime or getTime call on Mar 31");
     if (indt) {
         errln("Fail: Mar 31 inDaylightTime=TRUE, exp FALSE");
@@ -350,7 +358,7 @@ UBool
 TimeZoneRegressionTest::checkCalendar314(GregorianCalendar *testCal, TimeZone *testTZ) 
 {
     UErrorCode status = U_ZERO_ERROR;
-    // GregorianCalendar testCal = (GregorianCalendar)aCal.clone(); 
+    // GregorianCalendar testCal = aCal.clone(); 
 
     int32_t tzOffset, tzRawOffset; 
     float tzOffsetFloat,tzRawOffsetFloat; 
@@ -395,7 +403,12 @@ TimeZoneRegressionTest::checkCalendar314(GregorianCalendar *testCal, TimeZone *t
     UDate testDate = testCal->getTime(status); 
 
     UBool inDaylightTime = testTZ->inDaylightTime(testDate, status); 
-    SimpleDateFormat *sdf = new SimpleDateFormat((UnicodeString)"MM/dd/yyyy HH:mm", status); 
+    LocalPointer<SimpleDateFormat> sdf(new SimpleDateFormat((UnicodeString) "MM/dd/yyyy HH:mm", status),
+                                       status);
+    if (U_FAILURE(status)) {
+        errln("Error creating SimpleDateFormat %s", u_errorName(status));
+        return false;
+    }
     sdf->setCalendar(*testCal); 
     UnicodeString inDaylightTimeString; 
 
@@ -430,7 +443,6 @@ TimeZoneRegressionTest::checkCalendar314(GregorianCalendar *testCal, TimeZone *t
     else 
         errln(output);
 
-    delete sdf;
     return passed;
 } 
 
@@ -505,8 +517,8 @@ void TimeZoneRegressionTest:: Test4151406() {
         //try {
             UErrorCode ec = U_ZERO_ERROR;
             int32_t count;
-            StringEnumeration* ids = TimeZone::createEnumeration(rawoffset);
-            if (ids == NULL) {
+            StringEnumeration* ids = TimeZone::createEnumerationForRawOffset(rawoffset, ec);
+            if (U_FAILURE(ec)) {
                 dataerrln("Fail: TimeZone::createEnumeration(rawoffset)");
                 continue;
             }
@@ -551,16 +563,18 @@ void TimeZoneRegressionTest:: Test4151429() {
 void TimeZoneRegressionTest:: Test4154537() {
     UErrorCode status = U_ZERO_ERROR;
     // tz1 and tz2 have no DST and different rule parameters
-    SimpleTimeZone *tz1 = new SimpleTimeZone(0, "1", 0, 0, 0, 0, 2, 0, 0, 0, status);
-    SimpleTimeZone *tz2 = new SimpleTimeZone(0, "2", 1, 0, 0, 0, 3, 0, 0, 0, status);
+    LocalPointer<SimpleTimeZone> tz1(new SimpleTimeZone(0, "1", 0, 0, 0, 0, 2, 0, 0, 0, status), status);
+    LocalPointer<SimpleTimeZone> tz2(new SimpleTimeZone(0, "2", 1, 0, 0, 0, 3, 0, 0, 0, status), status);
     // tza and tzA have the same rule params
-    SimpleTimeZone *tza = new SimpleTimeZone(0, "a", 0, 1, 0, 0, 3, 2, 0, 0, status);
-    SimpleTimeZone *tzA = new SimpleTimeZone(0, "A", 0, 1, 0, 0, 3, 2, 0, 0, status);
+    LocalPointer<SimpleTimeZone> tza(new SimpleTimeZone(0, "a", 0, 1, 0, 0, 3, 2, 0, 0, status), status);
+    LocalPointer<SimpleTimeZone> tzA(new SimpleTimeZone(0, "A", 0, 1, 0, 0, 3, 2, 0, 0, status), status);
     // tzb differs from tza
-    SimpleTimeZone *tzb = new SimpleTimeZone(0, "b", 0, 1, 0, 0, 3, 1, 0, 0, status);
+    LocalPointer<SimpleTimeZone> tzb(new SimpleTimeZone(0, "b", 0, 1, 0, 0, 3, 1, 0, 0, status), status);
     
-    if(U_FAILURE(status))
-        errln("Couldn't create TimeZones");
+    if (U_FAILURE(status)) {
+        errln("Couldn't create TimeZones %s", u_errorName(status));
+        return;
+    }
 
     if (tz1->useDaylightTime() || tz2->useDaylightTime() ||
         !tza->useDaylightTime() || !tzA->useDaylightTime() ||
@@ -575,12 +589,6 @@ void TimeZoneRegressionTest:: Test4154537() {
         //errln("zone 1 = " + tz1);
         //errln("zone 2 = " + tz2);
     }
-
-    delete tz1;
-    delete tz2;
-    delete tza;
-    delete tzA;
-    delete tzb;
 }
 
 /**
@@ -618,7 +626,13 @@ void TimeZoneRegressionTest:: Test4154542()
         BAD,  UCAL_DECEMBER,  -32, -UCAL_SATURDAY,   0,
         BAD,  UCAL_DECEMBER,   31, -UCAL_SATURDAY-1, 0,
     };
-    SimpleTimeZone *zone = new SimpleTimeZone(0, "Z");
+    UErrorCode status = U_ZERO_ERROR;
+    LocalPointer<SimpleTimeZone> zone(new SimpleTimeZone(0, "Z"), status);
+    if (U_FAILURE(status)) {
+        errln("Fail: failed to create SimpleTimeZone %s", u_errorName(status));
+        return;
+    }
+
     for (int32_t i=0; i < 18*5; i+=5) {
         UBool shouldBeGood = (DATA[i] == GOOD);
         int32_t month     = DATA[i+1];
@@ -626,7 +640,7 @@ void TimeZoneRegressionTest:: Test4154542()
         int32_t dayOfWeek = DATA[i+3];
         int32_t time      = DATA[i+4];
 
-        UErrorCode status = U_ZERO_ERROR;
+        status = U_ZERO_ERROR;
 
         //Exception ex = null;
         //try {
@@ -663,6 +677,10 @@ void TimeZoneRegressionTest:: Test4154542()
                     (int8_t)month, (int8_t)day, (int8_t)dayOfWeek, time,
                     (int8_t)GOOD_MONTH, (int8_t)GOOD_DAY, (int8_t)GOOD_DAY_OF_WEEK, 
                     GOOD_TIME,status);
+        if (temp == nullptr) {
+            errln("Fail: failed to create SimpleTimeZone %s", u_errorName(status));
+            return;
+        }
         //} catch (IllegalArgumentException e) {
         //    ex = e;
         //}
@@ -681,6 +699,10 @@ void TimeZoneRegressionTest:: Test4154542()
                     (int8_t)GOOD_MONTH, (int8_t)GOOD_DAY, (int8_t)GOOD_DAY_OF_WEEK, 
                     GOOD_TIME,
                     (int8_t)month, (int8_t)day, (int8_t)dayOfWeek, time,status);
+        if (temp == nullptr) {
+            errln("Fail: failed to create SimpleTimeZone %s", u_errorName(status));
+            return;
+        }
         //} catch (IllegalArgumentException e) {
         //    ex = e;
         //}
@@ -692,7 +714,6 @@ void TimeZoneRegressionTest:: Test4154542()
         }            
         delete temp;
     }
-    delete zone;
 }
 
 
@@ -709,10 +730,10 @@ TimeZoneRegressionTest::Test4154525()
     int32_t DATA [] = {
         1, GOOD,
         0, BAD,
-        -1, BAD,
+        -1, GOOD,   // #13566 updates SimpleTimeZone to support negative DST saving amount
         60*60*1000, GOOD,
-        INT32_MIN, BAD,
-        // Integer.MAX_VALUE, ?, // no upper limit on DST savings at this time
+        INT32_MAX, GOOD,    // no upper limit on DST savings at this time
+        INT32_MIN, GOOD     // no lower limit as well
     };
 
     UErrorCode status = U_ZERO_ERROR;
@@ -721,19 +742,21 @@ TimeZoneRegressionTest::Test4154525()
         UBool valid = DATA[i+1] == GOOD;
         UnicodeString method;
         for(int32_t j=0; j < 2; ++j) {
-            SimpleTimeZone *z=NULL;
+            LocalPointer<SimpleTimeZone> z;
             switch (j) {
                 case 0:
                     method = "constructor";
-                    z = new SimpleTimeZone(0, "id",
+                    z.adoptInsteadAndCheckErrorCode(new SimpleTimeZone(0, "id",
                         UCAL_JANUARY, 1, 0, 0,
                         UCAL_MARCH, 1, 0, 0,
-                        savings, status); // <- what we're interested in
+                        savings, status), status); // <- what we're interested in
                     break;
                 case 1:
                     method = "setDSTSavings()";
-                    z = new SimpleTimeZone(0, "GMT");
-                    z->setDSTSavings(savings, status);
+                    z.adoptInsteadAndCheckErrorCode(new SimpleTimeZone(0, "GMT"), status);
+                    if (z.isValid()) {
+                        z->setDSTSavings(savings, status);
+                    }
                     break;
             }
 
@@ -754,7 +777,6 @@ TimeZoneRegressionTest::Test4154525()
                 }
             }
             status = U_ZERO_ERROR;
-            delete z;
         }
     }
 }
@@ -840,17 +862,21 @@ void
 TimeZoneRegressionTest::Test4162593() 
 {
     UErrorCode status = U_ZERO_ERROR;
-    SimpleDateFormat *fmt = new SimpleDateFormat("z", Locale::getUS(), status);
-    if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
-      delete fmt;
+    LocalPointer<SimpleDateFormat> fmt(new SimpleDateFormat("z", Locale::getUS(), status), status);
+    if (U_FAILURE(status)) {
+      dataerrln("Error creating SimpleDateFormat %s", u_errorName(status));
       return;
     }
     const int32_t ONE_HOUR = 60*60*1000;
 
-    SimpleTimeZone *asuncion = new SimpleTimeZone(-4*ONE_HOUR, "America/Asuncion" /*PY%sT*/,
+    LocalPointer<SimpleTimeZone> asuncion(new SimpleTimeZone(-4*ONE_HOUR, "America/Asuncion" /*PY%sT*/,
         UCAL_OCTOBER, 1, 0 /*DOM*/, 0*ONE_HOUR,
-        UCAL_MARCH, 1, 0 /*DOM*/, 0*ONE_HOUR, 1*ONE_HOUR, status);
+        UCAL_MARCH, 1, 0 /*DOM*/, 0*ONE_HOUR, 1*ONE_HOUR, status), status);
+
+    if (U_FAILURE(status)) {
+        dataerrln("Error creating SimpleTimeZone %s", u_errorName(status));
+        return;
+    }
 
     /* Zone
      * Starting time
@@ -866,10 +892,10 @@ TimeZoneRegressionTest::Test4162593()
         {2000, UCAL_FEBRUARY, 29, 22, 0},
      };
 
-    UBool DATA_BOOL [] = {
-        TRUE,
-        FALSE,
-        TRUE,
+    bool DATA_BOOL [] = {
+        true,
+        false,
+        true,
     };
     
     UnicodeString zone [4];// = new String[4];
@@ -877,7 +903,12 @@ TimeZoneRegressionTest::Test4162593()
         new SimpleTimeZone(2*ONE_HOUR, "Asia/Damascus" /*EE%sT*/,
             UCAL_APRIL, 1, 0 /*DOM*/, 0*ONE_HOUR,
             UCAL_OCTOBER, 1, 0 /*DOM*/, 0*ONE_HOUR, 1*ONE_HOUR, status);
-    DATA_TZ[1] = asuncion;  DATA_TZ[2] = asuncion;  
+    DATA_TZ[1] = asuncion.getAlias();  DATA_TZ[2] = asuncion.getAlias();
+
+    if (DATA_TZ[0] == nullptr || U_FAILURE(status)) {
+        errln("Error creating DATA_TZ[0] %s", u_errorName(status));
+        return;
+    }
 
     for(int32_t j = 0; j < 3; j++) {
         TimeZone *tz = (TimeZone*)DATA_TZ[j];
@@ -887,7 +918,7 @@ TimeZoneRegressionTest::Test4162593()
         // Must construct the Date object AFTER setting the default zone
         int32_t *p = (int32_t*)DATA_INT[j];
         UDate d = CalendarRegressionTest::makeDate(p[0], p[1], p[2], p[3], p[4]);
-       UBool transitionExpected = DATA_BOOL[j];
+        bool transitionExpected = DATA_BOOL[j];
 
         UnicodeString temp;
         logln(tz->getID(temp) + ":");
@@ -907,8 +938,6 @@ TimeZoneRegressionTest::Test4162593()
             errln("Fail: boundary transition incorrect");
         }
     }
-    delete fmt;
-    delete asuncion;
     delete DATA_TZ[0];
 }
 
@@ -920,26 +949,26 @@ void TimeZoneRegressionTest::Test4176686() {
     // that does have a DST savings (which should be ignored).
     UErrorCode status = U_ZERO_ERROR;
     int32_t offset = 90 * 60000; // 1:30
-    SimpleTimeZone* z1 = new SimpleTimeZone(offset, "_std_zone_");
-    z1->setDSTSavings(45 * 60000, status); // 0:45
+    SimpleTimeZone z1(offset, "_std_zone_");
+    z1.setDSTSavings(45 * 60000, status); // 0:45
 
     // Construct a zone that observes DST for the first 6 months.
-    SimpleTimeZone* z2 = new SimpleTimeZone(offset, "_dst_zone_");
-    z2->setDSTSavings(45 * 60000, status); // 0:45
-    z2->setStartRule(UCAL_JANUARY, 1, 0, status);
-    z2->setEndRule(UCAL_JULY, 1, 0, status);
+    SimpleTimeZone z2(offset, "_dst_zone_");
+    z2.setDSTSavings(45 * 60000, status); // 0:45
+    z2.setStartRule(UCAL_JANUARY, 1, 0, status);
+    z2.setEndRule(UCAL_JULY, 1, 0, status);
 
     // Also check DateFormat
-    DateFormat* fmt1 = new SimpleDateFormat(UnicodeString("z"), status);
+    SimpleDateFormat fmt1(UnicodeString(u"z"), status);
     if (U_FAILURE(status)) {
         dataerrln("Failure trying to construct: %s", u_errorName(status));
         return;
     }
-    fmt1->setTimeZone(*z1); // Format uses standard zone
-    DateFormat* fmt2 = new SimpleDateFormat(UnicodeString("z"), status);
+    fmt1.setTimeZone(z1); // Format uses standard zone
+    SimpleDateFormat fmt2(UnicodeString(u"z"), status);
     if(!assertSuccess("trying to construct", status))return;
-    fmt2->setTimeZone(*z2); // Format uses DST zone
-    Calendar* tempcal = Calendar::createInstance(status);
+    fmt2.setTimeZone(z2); // Format uses DST zone
+    LocalPointer<Calendar> tempcal(Calendar::createInstance(status));
     tempcal->clear();
     tempcal->set(1970, UCAL_FEBRUARY, 1);
     UDate dst = tempcal->getTime(status); // Time in DST
@@ -949,26 +978,26 @@ void TimeZoneRegressionTest::Test4176686() {
     // Description, Result, Expected Result
     UnicodeString a,b,c,d,e,f,g,h,i,j,k,l;
     UnicodeString DATA[] = {
-        "z1->getDisplayName(false, SHORT)/std zone",
-        z1->getDisplayName(FALSE, TimeZone::SHORT, a), "GMT+1:30",
-        "z1->getDisplayName(false, LONG)/std zone",
-        z1->getDisplayName(FALSE, TimeZone::LONG, b), "GMT+01:30",
-        "z1->getDisplayName(true, SHORT)/std zone",
-        z1->getDisplayName(TRUE, TimeZone::SHORT, c), "GMT+1:30",
-        "z1->getDisplayName(true, LONG)/std zone",
-        z1->getDisplayName(TRUE, TimeZone::LONG, d ), "GMT+01:30",
-        "z2->getDisplayName(false, SHORT)/dst zone",
-        z2->getDisplayName(FALSE, TimeZone::SHORT, e), "GMT+1:30",
-        "z2->getDisplayName(false, LONG)/dst zone",
-        z2->getDisplayName(FALSE, TimeZone::LONG, f ), "GMT+01:30",
-        "z2->getDisplayName(true, SHORT)/dst zone",
-        z2->getDisplayName(TRUE, TimeZone::SHORT, g), "GMT+2:15",
-        "z2->getDisplayName(true, LONG)/dst zone",
-        z2->getDisplayName(TRUE, TimeZone::LONG, h ), "GMT+02:15",
-        "DateFormat.format(std)/std zone", fmt1->format(std, i), "GMT+1:30",
-        "DateFormat.format(dst)/std zone", fmt1->format(dst, j), "GMT+1:30",
-        "DateFormat.format(std)/dst zone", fmt2->format(std, k), "GMT+1:30",
-        "DateFormat.format(dst)/dst zone", fmt2->format(dst, l), "GMT+2:15",
+        "z1.getDisplayName(false, SHORT)/std zone",
+        z1.getDisplayName(FALSE, TimeZone::SHORT, a), "GMT+1:30",
+        "z1.getDisplayName(false, LONG)/std zone",
+        z1.getDisplayName(FALSE, TimeZone::LONG, b), "GMT+01:30",
+        "z1.getDisplayName(true, SHORT)/std zone",
+        z1.getDisplayName(TRUE, TimeZone::SHORT, c), "GMT+1:30",
+        "z1.getDisplayName(true, LONG)/std zone",
+        z1.getDisplayName(TRUE, TimeZone::LONG, d ), "GMT+01:30",
+        "z2.getDisplayName(false, SHORT)/dst zone",
+        z2.getDisplayName(FALSE, TimeZone::SHORT, e), "GMT+1:30",
+        "z2.getDisplayName(false, LONG)/dst zone",
+        z2.getDisplayName(FALSE, TimeZone::LONG, f ), "GMT+01:30",
+        "z2.getDisplayName(true, SHORT)/dst zone",
+        z2.getDisplayName(TRUE, TimeZone::SHORT, g), "GMT+2:15",
+        "z2.getDisplayName(true, LONG)/dst zone",
+        z2.getDisplayName(TRUE, TimeZone::LONG, h ), "GMT+02:15",
+        "DateFormat.format(std)/std zone", fmt1.format(std, i), "GMT+1:30",
+        "DateFormat.format(dst)/std zone", fmt1.format(dst, j), "GMT+1:30",
+        "DateFormat.format(std)/dst zone", fmt2.format(std, k), "GMT+1:30",
+        "DateFormat.format(dst)/dst zone", fmt2.format(dst, l), "GMT+2:15",
     };
 
     for (int32_t idx=0; idx<UPRV_LENGTHOF(DATA); idx+=3) {
@@ -976,11 +1005,6 @@ void TimeZoneRegressionTest::Test4176686() {
             errln("FAIL: " + DATA[idx] + " -> " + DATA[idx+1] + ", exp " + DATA[idx+2]);
         }
     }
-    delete z1;
-    delete z2;
-    delete fmt1;
-    delete fmt2;
-    delete tempcal;
 }
 
 /**
@@ -1077,27 +1101,27 @@ TimeZoneRegressionTest::TestJDK12API()
     // TimeZone *cst1 = TimeZone::createTimeZone("CST");
     UErrorCode ec = U_ZERO_ERROR;
     //d,-28800,3,1,-1,120,w,9,-1,1,120,w,60
-    TimeZone *pst = new SimpleTimeZone(-28800*U_MILLIS_PER_SECOND,
+    LocalPointer<TimeZone> pst(new SimpleTimeZone(-28800*U_MILLIS_PER_SECOND,
                                        "PST",
                                        3,1,-1,120*U_MILLIS_PER_MINUTE,
                                        SimpleTimeZone::WALL_TIME,
                                        9,-1,1,120*U_MILLIS_PER_MINUTE,
                                        SimpleTimeZone::WALL_TIME,
-                                       60*U_MILLIS_PER_MINUTE,ec);
+                                       60*U_MILLIS_PER_MINUTE,ec), ec);
     //d,-21600,3,1,-1,120,w,9,-1,1,120,w,60
-    TimeZone *cst1 = new SimpleTimeZone(-21600*U_MILLIS_PER_SECOND,
+    LocalPointer<TimeZone> cst1(new SimpleTimeZone(-21600*U_MILLIS_PER_SECOND,
                                        "CST",
                                        3,1,-1,120*U_MILLIS_PER_MINUTE,
                                        SimpleTimeZone::WALL_TIME,
                                        9,-1,1,120*U_MILLIS_PER_MINUTE,
                                        SimpleTimeZone::WALL_TIME,
-                                       60*U_MILLIS_PER_MINUTE,ec);
+                                       60*U_MILLIS_PER_MINUTE,ec), ec);
     if (U_FAILURE(ec)) {
         errln("FAIL: SimpleTimeZone constructor");
         return;
     }
 
-    SimpleTimeZone *cst = dynamic_cast<SimpleTimeZone *>(cst1);
+    SimpleTimeZone *cst = dynamic_cast<SimpleTimeZone *>(cst1.getAlias());
 
     if(pst->hasSameRules(*cst)) {
         errln("FAILURE: PST and CST have same rules");
@@ -1130,9 +1154,6 @@ TimeZoneRegressionTest::TestJDK12API()
     if(savings != 60*60*1000) {
         errln("setDSTSavings() failed");
     }
-
-    delete pst;
-    delete cst;
 }
 /**
  * SimpleTimeZone allows invalid DOM values.
@@ -1206,4 +1227,61 @@ void TimeZoneRegressionTest::Test4184229() {
     delete zone;
 }
 
+void TimeZoneRegressionTest::TestNegativeDaylightSaving() {
+    UErrorCode status = U_ZERO_ERROR;
+    int32_t stdOff = 1 * 60*60*1000;    // Standard offset UTC+1
+    int save = -1 * 60*60*1000;     // DST saving amount -1 hour
+    SimpleTimeZone stzDublin(stdOff, "Dublin-2018",
+                                UCAL_OCTOBER, -1, -UCAL_SUNDAY, 2*60*60*1000,
+                                UCAL_MARCH, -1, -UCAL_SUNDAY, 1*60*60*1000,
+                                save, status);
+    failure(status, "SimpleTimeZone constructor");
+
+    if (save != stzDublin.getDSTSavings()) {
+        errln((UnicodeString)"FAIL: DST saving is not " + save);
+    }
+
+    GregorianCalendar cal(* TimeZone::getGMT(), status);
+    failure(status, "GregorianCalendar constructor");
+
+    UDate testDate;
+    int32_t rawOffset;
+    int32_t dstOffset;
+
+    cal.set(2018, UCAL_JANUARY, 15, 0, 0, 0);
+    testDate = cal.getTime(status);
+    failure(status, "calendar getTime() - Jan 15");
+
+    if (!stzDublin.inDaylightTime(testDate, status)) {
+        errln("FAIL: The test date (Jan 15) must be in DST.");
+    }
+    failure(status, "inDaylightTime() - Jan 15");
+
+    stzDublin.getOffset(testDate, FALSE, rawOffset, dstOffset, status);
+    failure(status, "getOffset() - Jan 15");
+    if (rawOffset != stdOff || dstOffset != save) {
+        errln((UnicodeString)"FAIL: Expected [stdoff=" + stdOff + ",save=" + save
+            + "] on the test date (Jan 15), actual[stdoff=" + rawOffset
+            + ",save=" + dstOffset + "]");
+    }
+
+    cal.set(2018, UCAL_JULY, 15, 0, 0, 0);
+    testDate = cal.getTime(status);
+    failure(status, "calendar getTime() - Jul 15");
+
+    if (stzDublin.inDaylightTime(testDate, status)) {
+        errln("FAIL: The test date (Jul 15) must be in DST.");
+    }
+    failure(status, "inDaylightTime() - Jul 15");
+
+    stzDublin.getOffset(testDate, FALSE, rawOffset, dstOffset, status);
+    failure(status, "getOffset() - Jul 15");
+    if (rawOffset != stdOff || dstOffset != 0) {
+        errln((UnicodeString)"FAIL: Expected [stdoff=" + stdOff + ",save=" + 0
+            + "] on the test date (Jul 15), actual[stdoff=" + rawOffset
+            + ",save=" + dstOffset + "]");
+    }
+}
+
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
