@@ -21,7 +21,7 @@
  **/
 class TimeZoneTest: public CalendarTimeZoneTest {
     // IntlTest override
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par ) override;
 public: // package
     static const int32_t millisPerHour;
  
@@ -99,6 +99,7 @@ public:
 
     void TestGetRegion(void);
     void TestGetUnknown();
+    void TestGetGMT();
 
     void TestGetWindowsID(void);
     void TestGetIDForWindowsID(void);
