@@ -683,7 +683,7 @@ public:
     virtual void getRules(
             const char *localeID, const char *collationType,
             UnicodeString &rules,
-            const char *&errorReason, UErrorCode &errorCode);
+            const char *&errorReason, UErrorCode &errorCode) override;
 
 private:
     const char *inputDir;
@@ -1191,7 +1191,7 @@ realParseTable(ParseState* state, TableResource *table, char *tag, uint32_t star
 
         if (token == TOK_CLOSE_BRACE)
         {
-            if (!readToken) {
+            if (!readToken && isVerbose()) {
                 warning(startline, "Encountered empty table");
             }
             return table;
